#ifndef __ASTROTOOLS_H
#define __ASTROTOOLS_H

#include <math.h>

typedef char bool;
#ifndef TRUE
#  define TRUE  (1)
#  define FALSE (0)
#endif
#ifndef true
#  define true  (1)
#  define false (0)
#endif

/* astronomische Konstanten */
#define JulDat2000 2451544.5   /* julianisches Datum des 1.1.2000, 0 UT */
#define JulDat1950 2433282.5   /* julianisches Datum des 1.1.1950, 0 UT */
#define JulDat1900 2415020.5   /* julianisches Datum des 1.1.1900, 0 UT */

typedef enum {
 	_UT 	= 0,
	_MEZ,
	_MESZ,
	_OEZ,
	_EST,
	_CST,
	_MST,
	_PST 
}	TTimeZone;

extern const double JulDatZeitDifferenz[10];
extern const char GetDatZoneStr[10][5];
extern const char SPACE[128];


typedef enum {	
	_TimeDisplay_Normal = 0,
	_TimeDisplay_Astro,
	_TimeDisplay_JD
}	TTimeDisplay;


extern TTimeZone    JulDatZone; /* gewhlte Zeitzone fr die JulDat-Funktionen */
extern TTimeDisplay JulDatShow; /* gewhlte Anzeigeart des Datums */


#define frac(r) 	( r<0 ? r+floor(-r) : r-floor(r) )
#define fracp(r)	( r<0 ? 1.0+r+floor(-r) : r-floor(r) )
#define trunc(d) 	((int)(floor(d)))
#define swapint(a,b)		{ int swap = a; a = b; b = swap; }
#define swapdouble(a,b)	{ double swap = a; a = b; b = swap; }
                        
char*		GetDateStr (char *dateStr);
char*		GetTimeStr (char *timeStr);
char*		GetYesterdayStr (char *dateStr);
double	GetTodaysJulDat();

int   	atoHell    (char *hell);

double 	Zeit2JulDat (char *datum);
char*		JulDat2Zeit (double jd, char *sz);

char*		Rekt2Str (double r, int secStellen, char *s);
char*		Dekl2Str (double d, int secStellen, char *s);

const char* Hell2Str( int hell100Mag );

char*		Winkel2HMS (double w, char *s);
char*		Winkel2Grad(double w, char *s);

int    	GetSpektrumNr(char *s);
char*		Spektrum(int sknr, char *s);

const char* Entfernung2Str (double distance);

extern int ConstellationMax;

const char* GetConstellationName ( int theConstellationN );
const char* GetConstellationShort( int theConstellationN );
const int 	GetConstellationMax  ();

#endif	//__ASTROTOOLS_H
