
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

#include "libtl_forms.h"
#include "libtl_postscript.h"
#include "libtl_printer.h"
#include "libtl_draw.h"
#include "libtl_string.h"


TDrawOutputFormat drawOutputFormat = DRAW_GIF;
int drawOutputWidth  = 640;
int drawOutputHeight = 480;

void draw_setup_format (TDrawOutputFormat theFormat)
{
	drawOutputFormat = theFormat;
}

void draw_setup_size (int theWidth, int theHeight)
{
	drawOutputWidth  = theWidth;
	drawOutputHeight = theHeight;
}


tstring drawCopyright = NULL;

void draw_setup_copyright( const char* theCopyright )
{
	SetString(&drawCopyright,theCopyright,0);
}


/****************************************************************************/
/* draw_XXX                                                                 */
/****************************************************************************/

PDrawBitmap draw_init (FL_OBJECT *ob, int *width, int *height, 
                       const char* pathname, const char *filename)
{
	static char *pic_ext[5] = {"gif","bmp","ppm","ps"};
	PDrawBitmap db = (PDrawBitmap) malloc(sizeof(DrawBitmap));
	if (ob!=NULL) 
	{
		db->offsetX = ob->x;
		db->offsetY = ob->y;
		*width  = ob->w;
		*height = ob->h;
		db->pic = NULL;
		db->textHeight = 16;
		db->textWidth  =  8;
		fl_set_clipping(ob->x,ob->y,ob->w,ob->h);
	}
	else 
	{
		char pic_name[512];
		sprintf(pic_name,"%s%s.%s",pathname,filename,pic_ext[drawOutputFormat]);
		*width  = drawOutputWidth;
		*height = drawOutputHeight;
		if (drawOutputFormat==DRAW_PS) 
		{
			db->pic = (void*) fopen(pic_name,"w");
		    	db->textHeight = 14;
   		 	db->textWidth  = 8;
			if (db->pic!=NULL)
			{	/* Header */
				ps_header((FILE*)db->pic,drawOutputWidth,filename,drawCopyright,db->textHeight);
			} 
			else
			{
				fprintf(stderr,"Kann '%s' nicht erzeugen\n",pic_name);
			}
		}
		else
		{
			db->offsetX = 0;
			db->offsetY = 0;
			db->pic = picture_init((long)drawOutputWidth,(long)drawOutputHeight,pic_name);
			picture_colormap_entry(db->pic,FL_WHITE ,0);
			picture_colormap_entry(db->pic,FL_BLACK ,RGB(255,255,255));
			picture_colormap_entry(db->pic,FL_RED   ,RGB(255,0,0));
			picture_colormap_entry(db->pic,FL_YELLOW,RGB(255,255,0));
			picture_colormap_entry(db->pic,FL_BLUE  ,RGB(0,0,255));
			picture_colormap_entry(db->pic,FL_GREEN ,RGB(0,255,0));
		 	db->textHeight = font_height;
			db->textWidth  = 8;
		}
	}
	return db;
}

void draw_free (PDrawBitmap db)
{
	if (db!=NULL) 
	{
		if (db->pic!=NULL) 
		{
			if (drawOutputFormat==DRAW_PS) 
			{
				ps_end((FILE*)db->pic);
			} else {
				picture_free(db->pic);
			}
		} 
		else
			fl_unset_clipping();
		free(db);
	}
	db = NULL;	
}

void draw_text (PDrawBitmap db, int x, int y, int c, const char *s) 
{
	if (db->pic==NULL)
		fl_drw_text(FL_ALIGN_LEFT,db->offsetX+x,db->offsetY+y,
		            db->textWidth*strlen(s),db->textHeight, 
		            c,10,FL_NORMAL_STYLE,(char*)s);
	else if (drawOutputFormat==DRAW_PS)
		ps_text((FILE*)db->pic,x,y+db->textHeight,0,s);
	else
		picture_text(db->pic,x,y,(Pixel)c,1,s);  	          
}

void draw_text_size (PDrawBitmap db, int x, int y, int c, int size, const char *s) 
{
	if (db->pic==NULL)
		fl_drw_text(FL_ALIGN_LEFT,db->offsetX+x,db->offsetY+y,
		            db->textWidth*strlen(s)*size,db->textHeight*size, 
		            c,10,FL_NORMAL_STYLE,(char*)s);
	else 
	if (drawOutputFormat==DRAW_PS) 
	{
		ps_text((FILE*)db->pic,x,y+db->textHeight,size,s);
	} 
	else 
	{
	  	picture_text(db->pic,x,y,(Pixel)c,size,s);  	          
	}
}

void draw_pixel (PDrawBitmap db, int x, int y, int c)
{
	draw_line(db,x,y,x,y,c);
}

void draw_line (PDrawBitmap db, int x, int y, int x1, int y1, int c)
{
	if (db->pic==NULL)
		fl_line(db->offsetX+x,db->offsetY+y,db->offsetX+x1,db->offsetY+y1,c);
	else if (drawOutputFormat==DRAW_PS)
		ps_line((FILE*)db->pic,x,y,x1,y1);
	else 
		picture_line(db->pic,x,y,x1,y1,(Pixel)c);  
}

void draw_rect (PDrawBitmap db, int x, int y, int x1, int y1, int c)
{
	if (db->pic==NULL) 
	{
		fl_line(db->offsetX+x,db->offsetY+y,db->offsetX+x1,db->offsetY+y,c);
		fl_line(db->offsetX+x1,db->offsetY+y,db->offsetX+x1,db->offsetY+y1,c);
		fl_line(db->offsetX+x1,db->offsetY+y1,db->offsetX+x,db->offsetY+y1,c);
		fl_line(db->offsetX+x,db->offsetY+y1,db->offsetX+x,db->offsetY+y,c);
	} 
	else 
	if (drawOutputFormat==DRAW_PS) 
	{
		ps_rect((FILE*)db->pic,x,y,x1,y1);
	} 
	else 
	{
		picture_line(db->pic,x,y,x1,y,(Pixel)c);
		picture_line(db->pic,x1,y,x1,y1,(Pixel)c);
		picture_line(db->pic,x1,y1,x,y1,(Pixel)c);
		picture_line(db->pic,x,y1,x,y,(Pixel)c);
	}
}

void draw_quad (PDrawBitmap db, int x, int y, int r, int c)
{
	r = (r+1)/2;
	draw_rect(db,x+r,y+r,x-r,y-r,c);
}

void draw_quad_fill (PDrawBitmap db, int x, int y, int r, int c)
{
	int x1,y1;
	r = (r+1)/2;
	x1 = x-r;
	y1 = y-r;
	x += r;
	y += r;
	if (db->pic==NULL) 
	{
		fl_line(db->offsetX+x,db->offsetY+y,db->offsetX+x1,db->offsetY+y,c);
		fl_line(db->offsetX+x1,db->offsetY+y,db->offsetX+x1,db->offsetY+y1,c);
		fl_line(db->offsetX+x1,db->offsetY+y1,db->offsetX+x,db->offsetY+y1,c);
		fl_line(db->offsetX+x,db->offsetY+y1,db->offsetX+x,db->offsetY+y,c);
	} 
	else 
	if (drawOutputFormat==DRAW_PS) 
	{
		ps_rect_fill((FILE*)db->pic,x,y,x1,y1,c);
	} 
	else 
	{
		picture_rect_fill(db->pic,x,y,x1,y1,(Pixel)c);
	}
}

void draw_circle (PDrawBitmap db, int x, int y, int r, int c)
{
	if (db->pic==NULL)
		fl_oval(0,db->offsetX+x,db->offsetY+y,r,r,c);
	else if (drawOutputFormat==DRAW_PS)
		ps_circle((FILE*)db->pic,x,y,r/2+1);
	else
		picture_circle(db->pic,x,y,r-1,(Pixel)c);  
}

void draw_circle_fill (PDrawBitmap db, int x, int y, int r, int c)
{
	if (db->pic==NULL)
		fl_oval(0,db->offsetX+x,db->offsetY+y,r,r,c);
	else if (drawOutputFormat==DRAW_PS)
		ps_circle_fill((FILE*)db->pic,x,y,r/2+1,c);
	else
		picture_circle_fill(db->pic,x,y,r-1,(Pixel)c);  
}

void draw_save (PDrawBitmap db)
{
	switch (drawOutputFormat) {
	case DRAW_GIF:
		write_picture_to_gif(db->pic,NULL);
		break;
	case DRAW_BMP:
		write_picture_to_bmp(db->pic,NULL);
		break;
	case DRAW_PPM:
		write_picture_to_ppm(db->pic,NULL);
		break;
	case DRAW_PS:
		/* nothing to do */
		break;
	}	
}

