#ifndef __LIBTL_PICTURE_H
#define __LIBTL_PICTURE_H

typedef unsigned char  Pixel;
typedef struct {
	Pixel         *data;
	long           width,height;
	long           x,y;
	char				name[256];					/* als File-Name gebraucht */
	unsigned char	colormap[4][256];
} Picture;

extern int            font_height;
extern unsigned char *font_data;

short    picture_font_load (const char *name);
void     picture_font_free (void);
Picture *picture_init (long width, long height, const char *name);
void     picture_free (Picture *pic);
void     picture_pixel (Picture *pic, long x, long y, Pixel color);
void     picture_horline (Picture *pic, long y, Pixel color);
void     picture_vertline (Picture *pic, long x, Pixel color);
void     picture_line (Picture *pic, long x0, long y0, long x1, long y1, Pixel color);
void     picture_rect (Picture *pic, long x0, long y0, long x1, long y1, Pixel color);
void     picture_rect_fill (Picture *pic, long x0, long y0, long x1, long y1, Pixel color);
void     picture_circle (Picture *pic, long x, long y, long radius, Pixel color);
void     picture_circle_fill (Picture *pic, long x, long y, long radius, Pixel color);
void     picture_vector (Picture *pic, long x0, long y0, long x1, long y1, Pixel color);
#define  picture_to(pic,x,y) { pic->x = x; pic->y = y; }
void     picture_lineto (Picture *pic, long x1, long y1, Pixel color);
void     picture_text (Picture *pic, long x, long y, Pixel color, int size, const char *text);
void     picture_shift_hor (Picture *pic, long shift, Pixel color);
void     picture_colormap_entry (Picture *pic, int nr, unsigned long color);
void     picture_colormap (Picture *pic, unsigned long *color);

void write_picture_to_bmp (Picture *pic, char *name);
void write_picture_to_gif (Picture *pic, char *name);
void write_picture_to_ppm (Picture *pic, char *name);

#endif /* __LIBTL_PICTURE_H */
