
#include <stdlib.h>
#include <math.h>

#include "vs_forms.h"

#include "libtl/libtl_string.h"
#include "libtl/libtl_printer.h"
#include "libtl/libtl_draw.h"

#include "vs_sub.h"
#include "vs_language.h"
#include "vs_file.h"
#include "vs_graph.h"

#ifndef NO_FORMS
FD_graf_phase 		*fd_graf_phase;
FD_graf_hell  		*fd_graf_hell;
FD_hell_attribut	*fd_hell_attr;
FD_bav_kurve  		*fd_bav_kurve;
FD_perana     		*fd_perana;
#else
FD_FORM  		*fd_graf_hell;
#endif

#define OBS_COLORS	5
#define OBS_FORMS 	2

const Pixel OBS_COLORTABLE[OBS_COLORS] = { FL_WHITE, FL_RED, FL_YELLOW, FL_BLUE, FL_GREEN };

void SymbolKleiner (PDrawBitmap db, int x, int y, Pixel color)
{
   draw_line(db,x-2,y-1,x+2,y-1,color);
   draw_line(db,x+2,y-1,x  ,y+2,color);
   draw_line(db,x  ,y+2,x-2,y-1,color);
}

#ifndef NO_FORMS
int HellPhase_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev);
int HellKurve_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev);
#endif


void RepaintGraph()
{
#ifndef NO_FORMS
	if (fd_graf_phase && fl_form_is_visible(fd_graf_phase->graf_phase))
	 	fl_redraw_form(fd_graf_phase->graf_phase);
	if (fd_graf_hell && fl_form_is_visible(fd_graf_hell->graf_hell))
	 	fl_redraw_form(fd_graf_hell->graf_hell);
#endif
}


/****************************************************************************/
/* Phasengrafik                                                             */
/****************************************************************************/

#ifndef NO_FORMS

void HellPhasePaint ()
{
	int           i,j,hmax,hmin,wx,wy,wb,wh,teiler;
	double        hjdmax,faky,phase;
	struct RHell *hellPtr;
	PDrawBitmap   db;
	char          str[80];

	wy = 10;
	db = draw_init(fd_graf_phase->grafik ,&wb,&wh, pfadAblage,diskLoad->name);
	if (db==NULL) return;
   wb = wb-50;
   wh = wh-wy-20;
   if (mitTheoretKurve) 
   {
      hmax = stern.hellMax;
      hmin = stern.hellMin;
   }
   else
   {
      hmax = 9999;
      hmin = -9999;
   }
   hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
    if (hellPtr->hell<10000) {
      if (hellPtr->hell<hmax) {
        hmax   = hellPtr->hell;
        hjdmax = hellPtr->datum;
       }
      if (hellPtr->hell>hmin) hmin = hellPtr->hell;
    }
    hellPtr = hellPtr->next;
  }
  teiler = (hmin-hmax<300) ? 50 : 100;
  hmax = (hmax/teiler)*teiler;
  hmin = (hmin/teiler)*teiler+teiler;
  if (stern.max_jd!=0) hjdmax = stern.max_jd;
  faky = (double)wh/(double)(hmin-hmax)*(double)teiler;
  for (i=0; i<(hmin-hmax)/teiler; i++) 
  {
    j = wy+(int)(i*faky);
    sprintf(str,"%5.1f",(hmax+i*teiler)*0.01);
    draw_text(db,2,j,FL_WHITE,str);
    draw_line(db,43,j,47,j,FL_WHITE);
    for (wx=0; wx<wb/5; wx++)
      draw_line(db,45+wx*5,j,46+wx*5,j,FL_WHITE);
  }
  wx = 45;
	draw_line(db,wx,5,wx,wy+wh,FL_WHITE);
	draw_line(db,wx,wy+wh,wx+wb,wy+wh,FL_WHITE);
  wy += wh;
  faky = (double)wh/(double)(hmin-hmax);
  for (i=0; i<5; i++) {
  	sprintf(str,"%2i",(i*25)%100);
    draw_text(db,37+(int)(i*wb*0.25),wy+5,FL_WHITE,str);
    draw_line(db,wx+(int)(i*wb*0.25),wy-2,wx+(int)(i*0.25*wb),wy+2,FL_WHITE);
    for (j=0; j<wh/5; j++)
      draw_line(db,wx+(int)(i*wb*0.25),wy-3-j*5,wx+(int)(i*0.25*wb),wy-4-j*5,FL_WHITE);
  }

  hellPtr = hellFirst;
  while (hellPtr!=NULL) {
    if ((besDatumVon<hellPtr->datum) && (hellPtr->datum<besDatumBis) && BitTest(besArt,hellPtr->art)) {
      phase = fracp((hellPtr->datum-hjdmax)/stern.periode);
      if (hellPtr->hell<10000) 
        draw_circle(db,wx+(int)(phase*wb),
                       wy-(int)((hmin-hellPtr->hell)*faky), 2,FL_YELLOW);
      else if (mitNegativBeob)
        SymbolKleiner(db,wx+(int)(phase*wb),
                         wy-(int)((hmin-hellPtr->hell+10000.0)*faky), FL_RED);
    }
    hellPtr = hellPtr->next;
  } 

  if (mitTheoretKurve) 
  {
    int x0,y0,x,y;
    x0 = wx;
    y0 = wy-(int)((hmin-HellJD(hjdmax))*faky);
    for (i=1; i<200; i++) {
      x = wx+(int)(i*0.005*wb);
      y = wy-(int)((hmin-HellJD(hjdmax+i*0.005*stern.periode))*faky);
      draw_line(db,x0,y0,x,y,FL_BLUE);
      x0=x;
      y0=y;
    }
  }

	draw_free(db);
}

void cb_HellPhase (FL_OBJECT *ob, long data)
{
   if (data==1) 
   {
      fd_graf_phase_wp.x = fd_graf_phase->graf_phase->x;
      fd_graf_phase_wp.y = fd_graf_phase->graf_phase->y;
      fd_graf_phase_wp.w = fd_graf_phase->graf_phase->w;
      fd_graf_phase_wp.h = fd_graf_phase->graf_phase->h;
      fl_hide_form(fd_graf_phase->graf_phase);
   }
}
	
int HellPhase_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
{
	switch (event)
	{
		case FL_DRAW:
			HellPhasePaint();
			break;
	}
	return 0;	
}

void HellPhase()
{
	char str[80];
	
	if ((stern.hellZahl==0) || (stern.periode<=0)) return;
	if ((fd_graf_phase!=NULL) && (fd_graf_phase->graf_phase->visible))
	{
		fl_hide_form(fd_graf_phase->graf_phase);
	}
	else
	{
		fd_graf_phase = create_form_graf_phase();	
		fl_set_form_minsize(fd_graf_phase->graf_phase,200,200);
		fl_set_object_posthandler( fd_graf_phase->grafik, HellPhase_Posthandler );
		fl_set_object_label(fd_graf_phase->Close, LS_Close );
		fl_set_object_shortcutkey( fd_graf_phase->Close, XK_Return );
	}
	sprintf(str,LS(165),stern.name);
	if (fd_graf_phase_wp.x>0) 
	{
		fl_set_form_position(fd_graf_phase->graf_phase, fd_graf_phase_wp.x,fd_graf_phase_wp.y);
		fl_set_form_size    (fd_graf_phase->graf_phase, fd_graf_phase_wp.w,fd_graf_phase_wp.h);
	}
   fl_show_form(fd_graf_phase->graf_phase,
                ((fd_graf_phase_wp.x>0) ? FL_PLACE_POSITION : FL_PLACE_MOUSE | FL_FREE_SIZE),
                FL_FULLBORDER,str);
	fl_raise_form(fd_graf_phase->graf_phase);
}

#endif

/****************************************************************************/
/* Helligkeitskurve                                                         */
/****************************************************************************/

double hell_jd0, hell_jd1, hell_avg;
int    hell_hmax, hell_hmin;
int    hell_mx,hell_my,hell_mx0,hell_mx1;
struct ROrganisation *hell_groupPtr[2] = {NULL,NULL};
bool   hell_groupAll    = TRUE;
int    hell_groupForm[2] = {1,2};
bool   hell_obsStd = TRUE;
tstring hell_obsName[HELL_OBS_N] = {NULL,NULL,NULL};
int    hell_obsForm[HELL_OBS_N+1]   = {3,4,5,7};

#define GRAF_HELL_LEFT  	40
#define GRAF_HELL_RIGHT 	10
#define GRAF_HELL_TOP		30
#define GRAF_HELL_BOTTOM	30

double GetJDGridStep( double jdDiff )
{
   double jdGridStep = jdDiff/10;
   if      (jdGridStep<0.05)	jdGridStep = 0.01;
   else if (jdGridStep<0.1)	jdGridStep = 0.05;
   else if (jdGridStep<0.15)	jdGridStep = 0.1;
   else if (jdGridStep<0.3)	jdGridStep = 0.25;
   else if (jdGridStep<0.6)	jdGridStep = 0.5;
   else if (jdGridStep<1.2)	jdGridStep = 1.0;
   else if (jdGridStep<6.0)	jdGridStep = 5.0;
   else if (jdGridStep<13.0)	jdGridStep = 10.0;
   else if (jdGridStep<30.0)	jdGridStep = 30.0;
   else if (jdGridStep<60.0)	jdGridStep = 50.0;
   else if (jdGridStep<130)	jdGridStep = 100.0;
   else if (jdGridStep<300)	jdGridStep = 250.0;
   else if (jdGridStep<600)	jdGridStep = 500.0;
   else if (jdGridStep<1300)	jdGridStep = 1000.0;
   else if (jdGridStep<6000)	jdGridStep = 5000.0;
   else jdGridStep = 10000;
   return jdGridStep;
}

void HellKurvePaint (int as_bmp, const char* theGraphFileName)
{
	int           i,d,x,y,x1,y1,teiler;
	int           wx,wy,wb,wh;
	double        jdif,hdif,j, jdGridStep, obsSum,obsCount;
	long          obsJD;
	struct RHell *hellPtr;
	PDrawBitmap   db;
	int numberOfObservers = 0;		// 25.03.01
	int nUnknownObserver = -1;
	int *countObsActivity = NULL;
	int hell_obsNameIndex[HELL_OBS_N];
	char          str[512];
	FILE*         fExport = NULL;

	/* Anpassen der Helligkeitsgrenzen */
	teiler = (hell_hmin-hell_hmax<300) ? 50 : 100;
	if (hell_hmin%teiler!=0)
		hell_hmin = (hell_hmin/teiler+1)*teiler;
	if (hell_hmax%teiler!=0) 
	{
		if ((hell_hmax%teiler>teiler/2) && (teiler==100))
			hell_hmax = (hell_hmax/teiler+1)*teiler;
		else
			hell_hmax = (hell_hmax/teiler)*teiler;
	}

	/* Ausmae der Grafik */
	wx = GRAF_HELL_LEFT;
	wy = GRAF_HELL_TOP;
	draw_setup_format( grafikDruckFormat );
	draw_setup_size( grafikDruckX, grafikDruckY );
	db = draw_init( as_bmp ? 0 : fd_graf_hell->grafik ,&wb,&wh, pfadAblage, 
	                theGraphFileName ? theGraphFileName : diskLoad->name );
	if (db==NULL) return;
	
	if (as_bmp==3 || as_bmp==4) 
	{	// 25.03.01
		// create array for observer counting
		RObserver *obsPtr = observerFirst;
		while (obsPtr) 
		{
			numberOfObservers++;
			obsPtr = obsPtr->next;
		}
		nUnknownObserver = numberOfObservers++;
		numberOfObservers++;
		countObsActivity = (int*) malloc( sizeof(int) * numberOfObservers );
		if (countObsActivity)
			memset( countObsActivity, 0, sizeof(int) * numberOfObservers );
		// create export file
		if (as_bmp==4) 
		{	// 19.07.03
			sprintf(str,"%s%s.lst", pfadAblage, theGraphFileName ? theGraphFileName : diskLoad->name );
		}
		else
		{
			sprintf(str,"%s%s_stat.txt", pfadAblage, diskLoad->name );
			fprintf(stderr,"Creating %s\n",str);
		}
		fExport = fopen(str,"w");
	}
	
	wb = wb-wx-GRAF_HELL_RIGHT;
	wh = wh-wy-GRAF_HELL_BOTTOM;
	draw_line(db,wx,wy,wx,wy+wh,FL_WHITE);
	draw_line(db,wx,wy+wh,wx+wb,wy+wh,FL_WHITE);
	draw_text(db,wx-12,wy-db->textHeight,FL_WHITE,"mag");

	d = (hell_hmin-hell_hmax)/teiler;
	for (i=0; i<d; i++) 
	{
		j = (double)i/(double)d;
		for (x=0; x<wb/5+1; x++)
			draw_line(db,wx+x*5,wy+(int)(wh*j),wx+x*5+1,wy+(int)(wh*j),FL_WHITE);
		draw_line(db,wx-2,wy+(int)(wh*j),wx+2,wy+(int)(wh*j),FL_WHITE);
		sprintf(str,"%4.1f",0.01*(hell_hmax+(hell_hmin-hell_hmax)*j));
		draw_text(db,5,wy+(int)(wh*j),FL_WHITE,str);
	}

	jdif = (double)wb/(hell_jd1-hell_jd0);
	hdif = (double)wh/(double)(hell_hmin-hell_hmax);

	jdGridStep = GetJDGridStep( hell_jd1-hell_jd0 );
	for (j=hell_jd0; j<=hell_jd1; j+= jdGridStep) 
	{
		x = wx+jdif*(j-hell_jd0);
		draw_line(db, x, wy+wh-2, x, wy+wh+2, FL_WHITE);
		for (y=0; y<wh/5; y++)
			draw_line(db, x,wy+y*5+1, x,wy+y*5, FL_WHITE);
	}
	sprintf(str,"deltaT=%g",jdGridStep);
	draw_text(db,5+wb/2-20,wy+wh+3,FL_WHITE,str);
	draw_line(db,wx,wy+wh-2,wx,wy+wh+2,FL_WHITE);
	ZeitDatum(hell_jd0,str);
	draw_text(db,5,wy+wh+3,FL_WHITE,str);
	draw_line(db,wx+wb,wy+wh-2,wx+wb,wy+wh+2,FL_WHITE);
	ZeitDatum(hell_jd1,str);
	draw_text(db,wx+wb-5*strlen(str),wy+wh+3,FL_WHITE,str);
	if (formDatum<2) 
	{
		sprintf(str,"%10.3f",hell_jd0);
		draw_text(db,5,wy+wh+db->textHeight+3,FL_WHITE,str);
		sprintf(str,"%10.3f",hell_jd1);
		draw_text(db,wx+wb-db->textWidth*strlen(str),wy+wh+db->textHeight+3,FL_WHITE,str);
	}
   
	if ((stern.periode!=0) && mitTheoretKurve) 
	{
		for (x=0; x<wb; x++) 
		{
			y = wy+(int)(hdif*(HellJD(hell_jd0+x/jdif)-hell_hmax));
			if (x==0) { x1=wx; y1=y; }
			else 
			{
				draw_line(db,x1,y1,wx+x,y,FL_YELLOW);
				x1 = wx+x;
				y1 = y;
		        }  
		}
	}

	obsCount = 0;
	obsSum = 0;
	obsJD = 0;
	for (i=0; i<HELL_OBS_N; i++)
	{
		if (hell_obsName[i] && hell_obsName[i][0])
				hell_obsNameIndex[i] = GetObserverIndex(hell_obsName[i]);
		else	hell_obsNameIndex[i] = -1;
	}

	hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
		int seenByGroupObsN = -1, seenByGroupObsN2 = -1;
		int seenByObs[HELL_OBS_N];
		bool seenByObsN = FALSE;
		bool isGroup1 = FALSE, isGroup2 = FALSE, isStdObs = FALSE;
		for (i=0; i<HELL_OBS_N; i++)	seenByObs[i] = -1;

		if (hell_groupPtr[0]!=NULL && IsObservationInOrg(hellPtr,hell_groupPtr[0],NULL,&seenByGroupObsN))
			isGroup1 = TRUE;
		if (hell_groupPtr[1]!=NULL && IsObservationInOrg(hellPtr,hell_groupPtr[1],NULL,&seenByGroupObsN2))
			isGroup2 = TRUE;
		for (i=0; i<HELL_OBS_N; i++)
			if (hell_obsNameIndex[i]>=0 && CanObserverOutput(hell_obsName[i],hellPtr))
			{
				seenByObs[i] = hell_obsNameIndex[i];
				seenByObsN = TRUE;
			}
		if (hell_obsStd)
			isStdObs = CanObserverOutput("",hellPtr) || BitTest(6,hellPtr->art);
		if ((hell_jd0<=hellPtr->datum) && (hellPtr->datum<=hell_jd1) 
		   &&  BitTest(besArt,hellPtr->art)
		   &&  (hell_groupAll || isGroup1 || isGroup2 || seenByObsN || isStdObs) )
		{
			bool paint = FALSE;
      	double paintJD;
      	int paintObs;
      	if (hell_avg>0) 
      	{
      		long jdn = hellPtr->datum / hell_avg;
      		if ( jdn > obsJD && hellPtr->hell < 10000) 
      		{
      			paint = (obsCount>0);
      			if (paint) 
      			{
	      			paintJD = hell_avg * obsJD;
   	   			paintObs = obsSum / obsCount;
   	   		}
      			obsJD = jdn;
      			obsSum = hellPtr->hell;
      			obsCount = 1;
      		} 
      		else
      		if ( jdn == obsJD ) 
      		{
      			obsSum += hellPtr->hell;
      			obsCount += 1;
      		}
      		if (hellPtr->next==NULL) 
      		{	// always paint the last observation
      			paint = TRUE;
      		}
      	}
      	else 
      	{
      		paint = TRUE;
      		paintJD = hellPtr->datum;
      		paintObs = hellPtr->hell;
      	}
      	if (paint) 
      	{
      		Pixel color; int form; int obsN = -1;
      		int mode = hell_groupForm[0];
      		if (isGroup2)			mode = hell_groupForm[1];
		for (i=0; i<HELL_OBS_N; i++)
      				if (seenByObs[i]>=0)
					mode = hell_obsForm[i];
      		if (isStdObs)			mode = hell_obsForm[HELL_OBS_N];
      		mode--;	// choice starts at 1
      		form = mode/OBS_COLORS;
      		color = OBS_COLORTABLE[mode%OBS_COLORS];
			if (paintObs<10000) 
			{
	         	switch (form) {
	         	case 0:
	   	         draw_circle_fill(db,wx+(int)(jdif*(paintJD-hell_jd0)),
  		   	                  wy+(int)(hdif*(paintObs-hell_hmax)),
     		   	               5,color);
     		   	   break;
	         	case 1:
	   	         draw_quad_fill  (db,wx+(int)(jdif*(paintJD-hell_jd0)),
  		   	                  wy+(int)(hdif*(paintObs-hell_hmax)),
     		   	               5,color);
     		   	   break;
     		   	}
	         } 
	         else
   	      if (mitNegativBeob)
      	      SymbolKleiner(db,wx+(int)(jdif*(paintJD-hell_jd0)),
         	                 wy+(int)(hdif*(paintObs-10000-hell_hmax)), color);
         	if (countObsActivity) 
         	{
					if (CanObserverOutput("",hellPtr)) {
						obsN = numberOfObservers-1;
					} else {
						obsN = (seenByGroupObsN>=0 ? seenByGroupObsN : (seenByGroupObsN2>=0 ? seenByGroupObsN2 : nUnknownObserver));
						for (i=0; i<HELL_OBS_N; i++)
      							if (seenByObs[i]>=0)
								obsN = seenByObs[i];
					}
					countObsActivity[obsN]++;
         	}
				if (fExport) 
				{	// TL 15.10.01
					fprintf(fExport,"%10.3f  %c%5.2f  %s\n", hellPtr->datum,
                               (hellPtr->hell<10000) ? ' ' : '<',
                               (hellPtr->hell%10000)*0.01,
                               (obsN==numberOfObservers-1 ? initialAAVSO : hellPtr->observers) );
         	}
			}
		}
		hellPtr = hellPtr->next;
	}

	if (as_bmp>0)
	{
#ifndef NO_FORMS
		if (fd_graf_hell)
			CURSOR_WATCH(fd_graf_hell->graf_hell->window);
#endif
		switch (as_bmp) { 
		case 1: {	/* Print */
			FILE *fLst;
			fLst = OpenPrinter();
			PrintScreen(fLst,db->pic->data,grafikDruckX,grafikDruckY,grafikDruckEinschubX,grafikDruckEinschubY);
			ClosePrinter(fLst);
			break; }	
		case 2:		/* Save */
			draw_save(db);
			break;
		case 3:		/* Export */
			if (countObsActivity && fExport) 
			{
				RObserver *obsPtr = observerFirst;
				int nTotal = 0;
				i = 0;
				while (obsPtr) 
				{
					if (countObsActivity[i]) 
					{
						fprintf(fExport,"%6i   %3.3s %s\n", countObsActivity[i], obsPtr->code, obsPtr->name );
						nTotal += countObsActivity[i];
					}
					i++;
					obsPtr = obsPtr->next;
				}
				if (countObsActivity[nUnknownObserver]>0) 
				{
					fprintf(fExport,"%6i   ? (unknown)\n", countObsActivity[nUnknownObserver] );
					nTotal += countObsActivity[nUnknownObserver];
				}
				fprintf(fExport,"%6i   Standard-Observer\n", countObsActivity[numberOfObservers-1] );
				nTotal += countObsActivity[numberOfObservers-1];
				fprintf(fExport,"----------\n%6i    Gesamt\n",nTotal);
			}
			break;
		}
#ifndef NO_FORMS
		if (fd_graf_hell)
			CURSOR_NORM(fd_graf_hell->graf_hell->window);
#endif
	}
	if (fExport)
	{
		free(countObsActivity);
		fclose(fExport);
	}
	draw_free(db);
}

void HellKurve_Close ()
{
#ifndef NO_FORMS
	fd_graf_hell_wp.x = fd_graf_hell->graf_hell->x;
	fd_graf_hell_wp.y = fd_graf_hell->graf_hell->y;
	fd_graf_hell_wp.w = fd_graf_hell->graf_hell->w;
	fd_graf_hell_wp.h = fd_graf_hell->graf_hell->h;
	fl_hide_form(fd_graf_hell->graf_hell);
#endif
}


const char *Help_GrafHell[] = 
	{"@_Taste \t Funktion",
	 "a   \t  Attribute der Beobachtungsdarstellung",
	 "d   \t  Grafik drucken",
	 "e   \t  End-Datum auf heutigen Tag setzen",
	 "h   \t  Helligkeitsachse an die Beobachtungen anpassen",
	 "n   \t  Negative Beobachtungen an/aus",
	 "q   \t  Fenster der Lichtkurve schliessen",
	 "r   \t  Rundet die Datumsgrenzen auf ganze Tage",
	 "s   \t  Grafik speichern in Datei <Sternname>.<Format>",
	 "t   \t  Theoretische Lichtkurve an/aus",
	 "u   \t  Unbekannte Beobachter an/aus",
	 "x   \t  Export der Beobachterliste",
	 "+-  \t  Zoom: Zeitachse halbieren / verdoppeln",
	 "up  \t  Die letzte Periode vor dem aktuellen Datum anzeigen",
	 "down \t Zeitachse ber alle Beobachtungen",
	 "left \t Datum nach links schieben",
	 "right\t Datum nach rechts schieben",
	 "",
/*	 "Mausbewegung \t zeigt Datum und Helligkeit an der Mausposition an", */
	 "linke Maustaste \t Auswahl eines neues Zeitabschnittes",
	 NULL 
	};
	 
const char *Help_GrafHell_e[] = 
	{"@_Key \t Function",
	 "a   \t  Attributes of the observation display",
	 "d   \t  Print",
	 "e   \t  Use today as right scale date",
	 "h   \t  Optimize brightness scale",
	 "n   \t  Switch negativ observations on/off",
	 "q   \t  Quit and close lightcurve windows",
	 "r   \t  Round date scales to noon",
	 "s   \t  Save graph in file <starname>.<format>",
	 "t   \t  Theoretical lightcurve on/off",
	 "u   \t  Unknown observers on/off",
	 "x   \t  Export observer statistics",
	 "+-  \t  Zoom: double or half time range",
	 "up  \t  Display the last period before today",
	 "down \t Display all observations (sets the time scale)",
	 "left \t Move timescale to the left",
	 "right\t Move timescale to the right",
	 "",
/*	 "Mausbewegung \t zeigt Datum und Helligkeit an der Mausposition an", */
	 "drag left mouse button \t Define a new time range",
	 NULL 
	};
	 

void HellKurve_Attribut_CB (FL_OBJECT *ob, long data)
{
#ifndef NO_FORMS
	char str[64];
	int i;
	int hell_group;

	strncpy(str, fl_get_input(fd_hell_attr->DateFirst), 40);
	if (*str!='\0')
		hell_jd0 = StrDate2JulDat(str);
	strncpy(str, fl_get_input(fd_hell_attr->DateLast), 40);
	if (*str!='\0')
		hell_jd1 = StrDate2JulDat(str);
	if (hell_jd1<hell_jd0) swapdouble(hell_jd0,hell_jd1);

	strncpy(str, fl_get_input(fd_hell_attr->HellMax), 40);
	if (*str!='\0')
		hell_hmax = (int)(atof(str)*100);
	strncpy(str, fl_get_input(fd_hell_attr->HellMin), 40);
	if (*str!='\0')
		hell_hmin = (int)(atof(str)*100);
	
	hell_avg = atof( fl_get_input(fd_hell_attr->DateAvg) );
	for (i=0; i<2; i++) 
	{
		hell_group = fl_get_choice( i==0 ? fd_hell_attr->SelectGroup : fd_hell_attr->SelectGroup2 );
		if (i==0) {
			hell_groupAll = (hell_group==1);
			if (hell_group>1)
				hell_group--;
		}
		if (hell_group<2) {
			hell_groupPtr[i] = NULL;
		} else {
			hell_groupPtr[i] = organisationFirst;
			hell_group--;
			while (--hell_group>0) {
				hell_groupPtr[i] = hell_groupPtr[i]->next;
			}
		}
	}

	for (i=0; i<2; i++) 
	{
		strncpy(str, fl_get_input( i==0 ? fd_hell_attr->SelectObs1 : fd_hell_attr->SelectObs2 ), 40);
		HellKurve_Beobachter( i, str, -1 );
	}

	hell_groupForm[0] = fl_get_choice(fd_hell_attr->ColorGroup1);
	hell_groupForm[1] = fl_get_choice(fd_hell_attr->ColorGroup2);
	hell_obsForm[0]   = fl_get_choice(fd_hell_attr->ColorObs1);
	hell_obsForm[1]   = fl_get_choice(fd_hell_attr->ColorObs2);
	hell_obsForm[HELL_OBS_N] = fl_get_choice(fd_hell_attr->ColorObsStd);
	hell_obsStd = fl_get_button(fd_hell_attr->DisplayStdObs);
	
	switch (data) {
	case 1: /* Close */
		fl_hide_form( fd_hell_attr->hell_attribut );
		break;
	case 2: /* Apply */
		break;
	}
	fl_redraw_form( fd_graf_hell->graf_hell );
#endif
}

void cb_HellKurve (FL_OBJECT *ob, long data)
{
#ifndef NO_FORMS
	const XEvent *xe;
	double        jd, periode;
	struct RHell *hellPtr;
	int           i;
		
	periode = stern.periode;
	if (periode<0)
		periode *= -3;
	else if (periode==0)
		periode = 365.0;
		
	switch (data) {
	case 1: /* Close */
		HellKurve_Close();
		break;
	case 2: /* Help */
		if (GetLanguage()==1)
			OpenHelp( LS(160), Help_GrafHell );
		else
			OpenHelp( LS(160), Help_GrafHell_e );
		break;	
	case 3: /* Print */
		HellKurvePaint(1,NULL);
		break;
	case 4: /* Save */
		HellKurvePaint(2,NULL);
		break;
	case 99: /* Taste */
		xe = fl_last_event();
		//fprintf(stderr,"shortcut type=%i state=%i keycode=%i sum=%i\n",
		//        xe->type,xe->xkey.state,xe->xkey.keycode,xe->xkey.state*256+xe->xkey.keycode);
		if (xe->type==3) 
		{
			switch (xe->xkey.state*256+xe->xkey.keycode) {
			case 38:	/* a */
				if (fd_hell_attr==NULL) 
				{
					struct ROrganisation *orgPtr = organisationFirst;
					fd_hell_attr = create_form_hell_attribut();
					fl_set_object_label( fd_hell_attr->TextDate0, LS(166) );
					fl_set_object_label( fd_hell_attr->TextDate1, LS(167) );
					fl_set_object_label( fd_hell_attr->TextAvg, LS(168) );
					fl_set_object_label( fd_hell_attr->TextTage, LS_Days );
					fl_set_object_label( fd_hell_attr->TextGroup , LS(164) );
					fl_set_object_label( fd_hell_attr->TextGroup2, LS(260) );
					fl_set_object_label( fd_hell_attr->TextObs1, LS(261) );
					fl_set_object_label( fd_hell_attr->TextObs2, LS(262) );
					fl_set_object_label( fd_hell_attr->TextApply, LS_Apply );
					fl_set_object_label( fd_hell_attr->TextOK, LS_OK );
					fl_clear_choice( fd_hell_attr->SelectGroup );
					fl_addto_choice( fd_hell_attr->SelectGroup, LS(163) );	// all
					fl_addto_choice( fd_hell_attr->SelectGroup, LS(263) );	// off
					fl_clear_choice( fd_hell_attr->SelectGroup2 );
					fl_addto_choice( fd_hell_attr->SelectGroup2, LS(263) );
					while (orgPtr) 
					{
						fl_addto_choice( fd_hell_attr->SelectGroup , orgPtr->code );
						fl_addto_choice( fd_hell_attr->SelectGroup2, orgPtr->code );
						orgPtr = orgPtr->next;
					}
					fl_clear_choice( fd_hell_attr->ColorGroup1 );
					fl_clear_choice( fd_hell_attr->ColorGroup2 );
					fl_clear_choice( fd_hell_attr->ColorObs1 );
					fl_clear_choice( fd_hell_attr->ColorObs2 );
					fl_clear_choice( fd_hell_attr->ColorObsStd );
					for (i=0; i<OBS_COLORS*OBS_FORMS; i++) 
					{
						fl_addto_choice( fd_hell_attr->ColorGroup1, LS(300+i) );
						fl_addto_choice( fd_hell_attr->ColorGroup2, LS(300+i) );
						fl_addto_choice( fd_hell_attr->ColorObs1  , LS(300+i) );
						fl_addto_choice( fd_hell_attr->ColorObs2  , LS(300+i) );
						fl_addto_choice( fd_hell_attr->ColorObsStd, LS(300+i) );
					}
					fl_set_choice(fd_hell_attr->ColorGroup1,hell_groupForm[0]);
					fl_set_choice(fd_hell_attr->ColorGroup2,hell_groupForm[1]);
					fl_set_choice(fd_hell_attr->ColorObs1  ,hell_obsForm[0]);
					fl_set_choice(fd_hell_attr->ColorObs2  ,hell_obsForm[1]);
					fl_set_choice(fd_hell_attr->ColorObsStd,hell_obsForm[HELL_OBS_N]);
					fl_set_object_label(fd_hell_attr->DisplayStdObs,LS(264));
					fl_set_button(fd_hell_attr->DisplayStdObs, hell_obsStd);
					fl_set_object_shortcutkey( fd_hell_attr->TextApply, XK_Return );
				}
				fl_show_form( fd_hell_attr->hell_attribut, FL_PLACE_CENTER, FL_FULLBORDER, LS(169) );
				break;
			case 40: /* d */
				HellKurvePaint(1,NULL);
				break;
			case 26: /* e */ {
				char date[32];
				hell_jd1 = trunc(Zeit2JulDat(GetDateStr(date))+1);
				break; }
			case 43: /* h */
				hell_hmax = 10000;
				hell_hmin = 0;
				hellPtr = hellFirst;
				while (hellPtr!=NULL) 
				{
					if ((hell_jd0<=hellPtr->datum) && (hellPtr->datum<=hell_jd1)) 
					{
						if (hellPtr->hell<hell_hmax) hell_hmax = hellPtr->hell;
						if (hellPtr->hell%10000>hell_hmin) hell_hmin = hellPtr->hell%10000;
					}
					hellPtr = hellPtr->next;
				}
				if (hell_hmax==10000) 
				{
					hell_hmax = stern.hellMax;
					hell_hmin = stern.hellMin;
				}  
				break;      	
			case 57: /* n */
				mitNegativBeob = !mitNegativBeob;
				break;
			case 24: /* q */
				HellKurve_Close();
				break;
			case 27: /* r */
				hell_jd0 = floor(hell_jd0);
				hell_jd1 = ceil(hell_jd1);
				break;				
			case 39: /* s */	 
				HellKurvePaint(2,NULL);
				break;
			case 28: /* t */
				mitTheoretKurve = !mitTheoretKurve;
				break;
			case 30: /* u */
				besArt ^= 1;
				break;
			case 53: /* x */
				HellKurvePaint(3,NULL);
				break;
			case 35: /* + */
			case 86: /* + numerik */
				jd = (hell_jd1 - hell_jd0)*0.25;
				hell_jd0 += jd;
				hell_jd1 -= jd;
				break;
			case 61: /* - */
			case 82: /* - numerik */
				jd = (hell_jd1 - hell_jd0)*0.5;
				hell_jd0 -= jd;
				hell_jd1 += jd;
				break;
			case 272: /* / */
			case 112: /* / numerik */
				break;
			case 291: /* * */
			case 63:  /* * numerik */
				break;
			case 98: /* up */
				hellPtr = hellFirst;
				while (hellPtr->next!=NULL) hellPtr = hellPtr->next;
				hell_jd1 = hellPtr->datum;
				hell_jd0 = hell_jd1 - stern.periode;
				break; 
			case 104: /* down */
				hell_jd0 = hellFirst->datum;
				hellPtr = hellFirst;
				while (hellPtr->next!=NULL) hellPtr = hellPtr->next;
				hell_jd1 = hellPtr->datum;
				if (hell_jd1-hell_jd0<stern.periode)
					hell_jd1 = hell_jd0 + stern.periode;
				break;	
			case 100: /* left */
				jd = (hell_jd1 - hell_jd0)*0.5;
				hell_jd0 -= jd;
				hell_jd1 -= jd;
				break;
			case 102: /* right */
				jd = (hell_jd1 - hell_jd0)*0.5;
				hell_jd0 += jd;
				hell_jd1 += jd;
				break;
			} /* eof switch (keycode) */
			fl_redraw_form(fd_graf_hell->graf_hell);
			/*HellKurvePaint(FALSE);*/
		} /* eof xe_type==3 */ 
		break;	
	}
#endif
}

#ifndef NO_FORMS
int HellKurve_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
{
	static char s[64], s1[32];
	static double faktorZeit, faktorHell;
	faktorZeit = (hell_jd1-hell_jd0)
	            /(double)(fd_graf_hell->grafik->w-GRAF_HELL_LEFT-GRAF_HELL_RIGHT); 
	faktorHell = (double)(hell_hmin-hell_hmax)*0.01
	            /(double)(fd_graf_hell->grafik->h-GRAF_HELL_TOP-GRAF_HELL_BOTTOM); 
		
	switch (event) {
	case FL_DRAW:
		HellKurvePaint(0,NULL);
		break;
	case FL_MOTION:
	case FL_MOUSE:
		hell_mx = mx;
		hell_my = my;
		ZeitDatum( hell_jd0+faktorZeit*(double)(mx-2-GRAF_HELL_LEFT), s );
		sprintf(s1,"%5.1f mag", (double)(my-2-GRAF_HELL_TOP)*faktorHell 
		                       +(double)hell_hmax*0.01 );
		strcat(s,"\n  "); strcat(s,s1);
		fl_set_object_label( fd_graf_hell->text, s );
		break;
	case FL_PUSH:	
		if (key==1) 
		{
			hell_mx0 = mx;
		}
		break;
	case FL_RELEASE:
		if ((key==1) && (hell_mx0!=mx)) 
		{
			hell_mx1 = mx;
			if (hell_mx0>hell_mx1) 
			{
				int swap;
				swap = hell_mx0; hell_mx0 = hell_mx1; hell_mx1 = swap;
			}
			hell_jd1 = hell_jd0 + faktorZeit * (double)(hell_mx1-GRAF_HELL_LEFT);
			hell_jd0 = hell_jd0 + faktorZeit * (double)(hell_mx0-GRAF_HELL_LEFT);
			fl_redraw_form(fd_graf_hell->graf_hell);
		}
		break;	
	}
	return 0;
}
#endif

void HellKurve_Reset()
{
	struct RHell *hellPtr;
	char str[80];

	if (stern.periode<0) 
	{	/* Nova */
		if (hellFirst!=NULL) 
		{
			hell_jd0 = hellFirst->datum;
			hellPtr = hellFirst;
			while (hellPtr->next!=NULL) hellPtr = hellPtr->next;
			hell_jd1 = hellPtr->datum;
			if (hell_jd0-hell_jd1>stern.periode)
				hell_jd1 = hell_jd0 - stern.periode;
		}
		else
		{
			hell_jd0 = stern.max_jd+stern.periode/10;
			hell_jd1 = Zeit2JulDat(GetDateStr(str)) +0.5;
 		}
	}
	else
	{	/* periodischer Stern */
		hell_jd1 = Zeit2JulDat(GetDateStr(str)) +0.5;
		hell_jd0 = hell_jd1 - stern.periode;
	}	
	hell_hmax = stern.hellMax;
	hell_hmin = stern.hellMin;
	hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
		if (hellPtr->hell<hell_hmax) hell_hmax = hellPtr->hell;
		if (hellPtr->hell%10000>hell_hmin) hell_hmin = hellPtr->hell%10000;
		hellPtr = hellPtr->next;
	}
	hell_avg = 0;	// start with single obs. display
}


void HellKurve_Datumsbereich( double jd0, double jd1 )
{
	hell_jd0 = jd0;
	hell_jd1 = jd1;
}

void HellKurve_Gruppe( int groupN, ROrganisation* groupPtr, int groupForm )
{
	if (groupN==-1)
	{
		hell_groupAll = (groupPtr!=NULL);
		return;
	}
	if (groupN<0 || groupN>1)
		return;
	hell_groupPtr[groupN] = groupPtr;
	if (groupForm>=0)
		hell_groupForm[groupN] = groupForm+1;
}

void HellKurve_Beobachter( int obsN, const char* observerCode, int observerForm )
{
	if (obsN==-1)
	{
		hell_obsStd = (observerCode!=NULL);
		if (observerForm>=0)
			hell_obsForm[HELL_OBS_N] = observerForm+1;
		return;
	}
	if (obsN<0 || obsN>HELL_OBS_N)
		return;
	if (observerCode && *observerCode!='\0')
		SetString( &hell_obsName[obsN], observerCode, 0 );
	else
		DeleteString( &hell_obsName[obsN] );
	if (observerForm>=0)
		hell_obsForm[obsN] = observerForm+1;
}

void HellKurve()
{
#ifndef NO_FORMS
	char str[80];

	if ((stern.hellZahl==0) && (stern.periode==0))
		return;
	if ((fd_graf_hell!=NULL) && (fd_graf_hell->graf_hell->visible))
		HellKurve_Close();
	else
	{
		fd_graf_hell = create_form_graf_hell();	
		fl_set_object_shortcut(fd_graf_hell->shortcut ,"adehnqrstux",FALSE);
		fl_set_object_shortcut(fd_graf_hell->shortcut1,"*/+-&A&B&C&D",FALSE);
		fl_set_object_posthandler( fd_graf_hell->grafik, HellKurve_Posthandler );
		fl_set_form_minsize(fd_graf_hell->graf_hell,450,200);
		fl_set_object_label( fd_graf_hell->Help, LS(9) );
		fl_set_object_label( fd_graf_hell->Print, LS_Print );
		fl_set_object_label( fd_graf_hell->Save, LS_Save );
		fl_set_object_label( fd_graf_hell->Close, LS_Close );
		fl_set_object_shortcutkey( fd_graf_hell->Close, XK_Return );
	}  	                                           

	HellKurve_Reset();

	sprintf(str,LS(161),stern.name);
   if (fd_graf_hell_wp.x>0) 
   {
		fl_set_form_position(fd_graf_hell->graf_hell, fd_graf_hell_wp.x,fd_graf_hell_wp.y);
		fl_set_form_size    (fd_graf_hell->graf_hell, fd_graf_hell_wp.w,fd_graf_hell_wp.h);
	}
   fl_show_form(fd_graf_hell->graf_hell,
                ((fd_graf_hell_wp.x>0) ? FL_PLACE_POSITION : FL_PLACE_MOUSE | FL_FREE_SIZE),
                FL_FULLBORDER,str);
   fl_raise_form(fd_graf_hell->graf_hell);
#endif
}

void Script_HellKurve ()
{
}

/****************************************************************************/
/* Analyse                                                                  */
/****************************************************************************/

#ifndef NO_FORMS

void Analyse_MaxDatum()
{
	int		d,ylast,m,schrittZahl;
	double	jd0,fs0,fs,ws,zeitFaktor,startZeit,endZeit;
	struct RHell *hellPtr;
	int 		maxima[5][2];
	char		str[256];
	
	fprintf(stderr,"Perioden-Analyse\n");
	startZeit = besDatumVon;
	endZeit   = besDatumBis;
	schrittZahl = 1000;
	if (schrittZahl<10) return;
	zeitFaktor = (endZeit-startZeit)/schrittZahl;

	fprintf(stderr,"Analyse der Beobachtungsdaten von %s\n",stern.name);
	/*
	Line(10,380,10,100);
	Line(10,380,510,380);
	*/
	for (m=0; m<5; m++) 
	{
		maxima[m][0] = 999;
		maxima[m][1] = 0;
	}

	jd0 = hellFirst->datum;
	for (d=0; d<schrittZahl; d++) 
	{
		double fsr = 0;
		double fsi = 0;
		if (d==0) ws = 0;
		else      ws = startZeit + (double)d * zeitFaktor;
		/*
		if (d%100==0) {
			if (d==0) OutTextXY(1+d,399,Real2Str(startZeit,5,2))
			else OutTextXY(1+d,399,Real2Str(ws,5,2));
			Line(10+d,380,10+d,384);
		}
		*/
		hellPtr = hellFirst;
		while (hellPtr!=NULL) 
		{
			if (hellPtr->hell<10000) 
			{
				double phase = ws*(hellPtr->datum-jd0);
				fsr += hellPtr->hell*cos(phase);
				fsi -= hellPtr->hell*sin(phase);
			}
			hellPtr = hellPtr->next;
		}
		fs = sqrt(fsr*fsr+fsi*fsi);
		if (d>0) 
		{
			//MoveTo(9+d,ylast);
			ylast = 380-trunc(280.0*fs/fs0);
			//LineTo(10+d,ylast);
			m = 4;
			while ((m>0) && (ylast<maxima[m][0])) 
			{
				if (m<5) 
				{
					maxima[m][0] = maxima[m-1][0];
					maxima[m][1] = maxima[m-1][1];
				}
				m--;
			}
			if (m<4) 
			{
				maxima[m][0] = ylast;
				maxima[m][1] = d;
				m++;
			}
		} 
		else 
		{
			ylast = 380;
			fs0 = fs;
		}
	}

	for (m=0; m<5; m++) 
	{
		double jd = (double)maxima[m][1]*zeitFaktor+startZeit;
		fprintf(stderr, "%i %i %f  %s\n", maxima[m][1], maxima[m][0]-10, jd, ZeitDatum(jd,str) );
	}
}

typedef struct {
	int     n;
	double  period0, period1, periodStep, minRms, bestPeriod;
	double *rms;
} AnaPerRmsT;

typedef struct {
	int     n;
	double  period;
	double *avg;
} AnaPerPhaseAvgT;

AnaPerRmsT 			anaPerRms = {0};
int					peranaDisplayPhase = 0;
AnaPerPhaseAvgT	anaPerPhase = {10,0,NULL};

void Analyse_Periode( AnaPerRmsT* ap, int avgStep )
{	// October 2000, TL
	double avgMag[100], avgRms[100];
	int    avgN[100], periodN, periodI;
	int    i;
	struct RHell *hellPtr;
	
	CURSOR_WATCH(fd_perana->perana->window);
	if (avgStep>100)	avgStep = 100;
	periodN = (ap->period1 - ap->period0) / ap->periodStep +1;
	if (periodN!=ap->n)
	{
		if (ap->rms!=NULL)	free( ap->rms );
		ap->rms = malloc( sizeof(double)*periodN );
		ap->n = periodN;
	}
	ap->minRms = 1e9;
	
	for (periodI=0; periodI<periodN; periodI++) 
	{
		double period = ap->period0 + ap->periodStep * periodI;
		// reset table of phase-shifted and averaged brightness
		for (i=0; i<avgStep; i++) 
		{
			avgMag[i] = 0;
			avgRms[i] = 0;
			avgN[i] = 0;
		}
		// calculate the table
		hellPtr = hellFirst;
		while (hellPtr) 
		{
			if (hellPtr->hell<10000) 
			{
				double jd = hellPtr->datum / period;
				i = fracp(jd) * avgStep;
				avgN[i]++;
				avgMag[i] += hellPtr->hell;
				avgRms[i] += hellPtr->hell * hellPtr->hell;
			}
			hellPtr = hellPtr->next;
		}
		// calculate the error
		ap->rms[periodI] = 0;
		for (i=0; i<avgStep; i++) 
		{
			if (avgN[i]>0) 
			{
				double error = avgRms[i] * avgN[i] - avgMag[i] * avgMag[i];
				error = sqrt(error) / avgN[i];
				avgMag[i] /= avgN[i];
				avgRms[i] = error;
				ap->rms[periodI] += error;
				//fprintf(stderr,"  %03i\t%g\t%g\n", i, avgMag[i], error );
			}
		}
		// use maximum error
		if (ap->rms[periodI] < ap->minRms) 
		{
			ap->minRms = ap->rms[periodI];
			ap->bestPeriod = ap->period0 + ap->periodStep * periodI;
		}
	}
	
	CURSOR_NORM(fd_perana->perana->window);
}

void Analyse_PeriodePhase( AnaPerPhaseAvgT* aph )
{	// October 2000, TL
	int    avgN[100], i;
	struct RHell *hellPtr;

	if (aph->period < 0.001)
		aph->period = 1;
	if (aph->n > 100)	
		aph->n = 100;	
	if (aph->n < 5)	
		aph->n = 5;	
	if (aph->n==0 || aph->avg==NULL) {
		if (aph->avg!=NULL)	free( aph->avg );
		aph->avg = malloc( sizeof(double)*aph->n );
	}
	
	// reset table of phase-shifted and averaged brightness
	for (i=0; i<aph->n; i++) {
		aph->avg[i] = 0;
		avgN[i] = 0;
	}

	// calculate the table
	hellPtr = hellFirst;
	while (hellPtr) {
		if (hellPtr->hell<10000) {
			double jd = hellPtr->datum / aph->period;
			i = fracp(jd) * aph->n;
			avgN[i]++;
			aph->avg[i] += hellPtr->hell;
		}
		hellPtr = hellPtr->next;
	}

	// calculate the average
	for (i=0; i<aph->n; i++)
		if (avgN[i]>0)
			aph->avg[i] /= (double)avgN[i];
		else
			aph->avg[i] = -1e6;
}

void AnaPerToFile( FILE* f, AnaPerRmsT* ap )
{
	int periodI;
	for (periodI=0; periodI<ap->n; periodI++) {
		double period = ap->period0 + ap->periodStep * periodI;
		fprintf(f,"%010f  %f\n", period, ap->minRms/ap->rms[periodI] );
	}
	fprintf(f,"best period= %10f\n", ap->bestPeriod);
}

void PerAna_Paint( AnaPerRmsT* ap, AnaPerPhaseAvgT* phaseAvg )
{
	PDrawBitmap   db;
	int           wx,wy,wb,wh;
	int           i, xl0,yl0, xl1,yl1;
	double        fx,fy;
	char          str[80];
		
	//fl_redraw_form(fd_perana->perana);
	if (ap->n<2)	return;
	wy = 10;
	db = draw_init( fd_perana->grafik ,&wb,&wh, pfadAblage,diskLoad->name);
	if (db==NULL) 	return;
	if (phaseAvg)	wh/=2;
	fx = (double)wb / (double)ap->n ;
	fy = (double)wh;

	for (i=0; i<ap->n; i++) {
		xl1 = fx * i;
		yl1 = (1.0 - ap->minRms/ap->rms[i]) * fy;
		if (i>0)
			draw_line( db, xl0,yl0, xl1,yl1, FL_WHITE );
		xl0 = xl1;
		yl0 = yl1;
	}
	sprintf(str,"%g",ap->bestPeriod);
	draw_text( db, 
	           (ap->bestPeriod - ap->period0) / (ap->period1 - ap->period0) * (double)wb, 10,
	           FL_RED, str );
	
	if (phaseAvg) {
		double minHell = 50000, maxHell = 0;
		for (i=0; i<phaseAvg->n; i++) {
			if (phaseAvg->avg[i]>maxHell)
				maxHell = phaseAvg->avg[i];
			if (phaseAvg->avg[i]<minHell && phaseAvg->avg[i]>-1e6)
				minHell = phaseAvg->avg[i];
		}
		fx = (double)wb / (double)phaseAvg->n ;
		fy = (double)wh / (maxHell-minHell);
		xl0 = -1;
		for (i=0; i<phaseAvg->n; i++) 
			if (phaseAvg->avg[i]>-1e6) {
				xl1 = fx * i;
				yl1 = wh + (maxHell - phaseAvg->avg[i]) * fy;
				if (xl0>=0)
					draw_line( db, xl0,yl0, xl1,yl1, FL_WHITE );
				xl0 = xl1;
				yl0 = yl1;
			}
		sprintf(str,"Period= %g days",phaseAvg->period);
		draw_text( db, 2, wh+10, FL_RED, str );
	}
	
	draw_free(db);
}

void PerAna_Close()
{
	fd_perana_wp.x = fd_perana->perana->x;
	fd_perana_wp.y = fd_perana->perana->y;
	fd_perana_wp.w = fd_perana->perana->w;
	fd_perana_wp.h = fd_perana->perana->h;
	fl_hide_form( fd_perana->perana );
}

int PerAna_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
{
//	static int setRangeX0, setRangeX1;
	
	switch (event) {
	case FL_DRAW:
		PerAna_Paint( &anaPerRms, (peranaDisplayPhase ? &anaPerPhase : NULL) );
		break;
	/*
	case FL_MOTION:
	case FL_MOUSE:
		hell_mx = mx;
		hell_my = my;
		ZeitDatum( hell_jd0+faktorZeit*(double)(mx-2-GRAF_HELL_LEFT), s );
		sprintf(s1,"%5.1f mag", (double)(my-2-GRAF_HELL_TOP)*faktorHell 
		                       +(double)hell_hmax*0.01 );
		strcat(s,"\n  "); strcat(s,s1);
		fl_set_object_label( fd_graf_hell->text, s );
		break;
	*/
	case FL_PUSH:	
		if (key==1) {
			if (peranaDisplayPhase) {
				double wb = fd_perana->grafik->w;
				mx -= fd_perana->grafik->x;
				anaPerPhase.period = anaPerRms.period0 + (double)mx/wb * (anaPerRms.period1-anaPerRms.period0);
				Analyse_PeriodePhase( &anaPerPhase );
				fl_redraw_form(fd_perana->perana);
			}
			//setRangeX0 = mx;
		}
		break;
	/*
	case FL_RELEASE:
		if ((key==1) && (setRangeX0!=mx)) {
			setRangeX1 = mx;
			if (setRangeX0>setRangeX1) 
				swapint(setRangeX0,setRangeX1); 
			fl_redraw_form(fd_perana->perana);
		}
		break;
	*/
	}
	return 0;
}

void cb_PerAna (FL_OBJECT *ob, long data)
{
   switch (data) {
   case 1:	// Close
   	PerAna_Close();
      break;
	case 2:	// Repaint
		anaPerRms.period0 = atof( fl_get_input(fd_perana->PeriodFirst) );
		anaPerRms.period1 = atof( fl_get_input(fd_perana->PeriodLast) );
		anaPerRms.periodStep = atof( fl_get_input(fd_perana->PeriodStep) );
		anaPerPhase.n = atoi( fl_get_input(fd_perana->PhaseSteps) );
		if (anaPerPhase.n<5)	{
			anaPerPhase.n = 5;
			fl_set_input(fd_perana->PhaseSteps,"5");
		}
		if (anaPerPhase.n>100) {
			anaPerPhase.n = 100;
			fl_set_input(fd_perana->PhaseSteps,"100");
		}
		Analyse_Periode( &anaPerRms, anaPerPhase.n );
		if (peranaDisplayPhase) {
			anaPerPhase.period = atof( fl_get_input( fd_perana->DisplayPeriod ) );
			Analyse_PeriodePhase( &anaPerPhase );
		}
		fl_redraw_form(fd_perana->perana);
		break;
	case 3:	// MaxSearch
		Analyse_MaxDatum();
		break;
	case 4:	// PhaseShift
		peranaDisplayPhase = fl_get_button(fd_perana->PhaseShift);
		if (peranaDisplayPhase) {
			char str[30];
			anaPerPhase.period = anaPerRms.bestPeriod;
			Analyse_PeriodePhase( &anaPerPhase );
			sprintf(str,"%g",anaPerPhase.period);
			fl_set_input( fd_perana->DisplayPeriod, str );
		}
		fl_redraw_form( fd_perana->perana );
		break;
	case 5:	// PhasePeriod
		anaPerPhase.period = atof( fl_get_input( fd_perana->DisplayPeriod ) );
		fl_redraw_form( fd_perana->perana );
		break;
   }
}

void Analyse()
{
	char str[128];
	if (stern.hellZahl<5)	return;
	
	if (fd_perana!=NULL) 
	{
		if (fd_perana->perana->visible)
			PerAna_Close();
	} 
	else 
	{
		anaPerRms.n = 0;
		anaPerRms.period0 = 5;
		anaPerRms.period1 = 1000;
		anaPerRms.periodStep = 5;
		anaPerRms.rms = NULL;
		fd_perana = create_form_perana();	
		fl_set_object_posthandler( fd_perana->grafik, PerAna_Posthandler );
		fl_set_object_label( fd_perana->PeriodFirst, LS(201) );
		fl_set_object_label( fd_perana->PeriodLast, LS(202) );
		fl_set_object_label( fd_perana->PeriodStep, LS(203) );
		fl_set_object_label( fd_perana->Repaint, LS(204) );
		fl_set_object_label( fd_perana->Close, LS_Close );
		fl_set_object_label( fd_perana->MaxSearch, LS(205) );
		fl_set_object_label( fd_perana->PhaseShift, LS(206) );
		fl_set_object_label( fd_perana->PhaseSteps, LS(208) );
		sprintf(str,"%g",anaPerRms.period0);
		fl_set_input(fd_perana->PeriodFirst, str);
		sprintf(str,"%g",anaPerRms.period1);
		fl_set_input(fd_perana->PeriodLast, str);
		sprintf(str,"%g",anaPerRms.periodStep);
		fl_set_input(fd_perana->PeriodStep, str);
		sprintf(str,"%i",anaPerPhase.n);
		fl_set_input(fd_perana->PhaseSteps, str);
		fl_set_button(fd_perana->PhaseShift, peranaDisplayPhase );
	}

	sprintf(	str, LS(200), stern.name );
	fl_show_form( fd_perana->perana,
                 ((fd_perana_wp.x>0) ? FL_PLACE_POSITION : FL_PLACE_MOUSE | FL_FREE_SIZE),
                 FL_FULLBORDER, str );
	fl_raise_form( fd_perana->perana );

	Analyse_Periode( &anaPerRms, anaPerPhase.n );
	//PerAna_Paint( &anaPerRms );
	//AnaPerToFile(stderr,&anaPerRms);
	fl_redraw_form(fd_perana->perana);
}

#endif // !NO_FORMS

/****************************************************************************/
/* BAV_HellKurve                                                            */
/****************************************************************************/
double bavhk_fak_zeit;  /* Pixel/Tag   */
double bavhk_fak_hell;  /* Pixel/Mag   */
double bavhk_jd, bavhk_dauer; /* Tagesbereich */
int    bavhk_hmax, bavhk_hmin; /* Helligkeitsbereich */
char   bavhk_kommentar[512];
int    bavhk_faltung = 0;/* Faltung an/aus */
double bavhk_periode;	/* Periode fr Faltungsmodus */
double bavhk_faltung_jd;/* Zentrum der Faltungsdarstellung */
double bavhk_faltungsphase;
int    bavhk_spiegel;   /* Beobachtungen bei der Faltung auch gespiegelt darstellen */
double bavhk_jd0;

#define BAVHK_MINX 450
#define BAVHK_MINY 300

#ifndef NO_FORMS

void BAV_HellKurve_Paint (bool as_bmp)
{
	int           	i,x,y, maxX,maxY, wx,wy,wb,wh, hzahl, textposNObsX, kommentarZeile=0;
	double        	hdif,xr,yr, pixelStepX,pixelStepY;
	double        	jdGridStep,jGrid;
	double       	jdd, jd0,jd1;
	struct RHell* 	hellPtr;
	char          	str[128];
	char*				ptr;
	TTimeZone     	oldZeitZone;
	int           	old_grafikDruckY;
	PDrawBitmap   	db;
	int            obsColor = (as_bmp==2 ? FL_BLUE : FL_WHITE);
	int            obsRadius = (as_bmp==1) ? 5 : (as_bmp==2 ? 4 : 3);
	int				kommentarN;
	int				kommentarMax[10];
	double			kommentarJD[10];
	int				kommentarJDunsicher[10];
	char				kommentarHell[10][10];
	char				kommentarBemerkung[10][20];

	oldZeitZone = JulDatZone;
	JulDatZone  = _UT;
	if (bavhk_faltung) 
	{
		jdd = bavhk_periode;
		jd0 = bavhk_jd0;
		jd1 = bavhk_jd;
	} 
	else 
	{
		jdd = bavhk_dauer;
		jd0 = bavhk_jd - jdd*0.5;
		jd1 = bavhk_jd + jdd*0.5;
	}

	switch (as_bmp) {
	case 1: /* Druck */
	 	old_grafikDruckY = grafikDruckY;
	 	grafikDruckY = 2*grafikDruckY+8;
	case 2:	/* Save */
		draw_setup_format( grafikDruckFormat );
		draw_setup_size( grafikDruckX, grafikDruckY );
		{	// TL 11.03.04: BAV File-Name
			char bavName[512], *ptr;
			int datum = 0;
			ptr = bavhk_kommentar;
			while (*ptr!=0 && !isdigit(*ptr))	ptr++;
			datum = atoi(ptr);
			datum = datum%100000;
			sprintf( bavName, "%.3s_%s_%i_%s", diskLoad->name, diskLoad->name+3, datum, initialBAV );
		 	db = draw_init(0 ,&wb,&wh, pfadAblage,bavName);
		}
		if (db==NULL) return;
		maxX = wb;
		maxY = wh;
		/* Text-Kopf erzeugen */
		strcpy(str,stern.name);
		ptr = strchr(str,'(');
		if (ptr!=NULL) *ptr='\0';
		draw_text_size(db,4,4,FL_RED,4,stern.name);
		wx = 4*strlen(stern.name)*db->textWidth;
		wy = 4*db->textHeight;
		draw_line(db,0,wy,wx,wy,FL_RED);
		draw_line(db,0,wy+3,wx,wy+3,FL_RED);
		wx = wx+2*db->textWidth;
		textposNObsX = wx;
		sprintf(str,"Beobachter: %s (%s)",bavObserver,initialBAV);
		draw_text(db,wx,db->textHeight,FL_RED,str);
		sprintf(str,"Karte: %s",stern.karte);
		draw_text(db,wx,db->textHeight*2,FL_RED,str);
		wx = 4*db->textWidth;
		wy = 6*db->textHeight;
		if (as_bmp==1)
			draw_rect(db,0,0,maxX,maxY,FL_WHITE);
	 	break;

	default: 	
		sprintf(str,LS(210),stern.name);
		fl_set_form_title( fd_bav_kurve->bav_kurve, str );		
	 	db = draw_init(fd_bav_kurve->Grafik ,&wb,&wh, NULL,NULL);
		if (db==NULL) return;
		maxX = wb;
		maxY = wh;
	 	wx = 4*db->textWidth;
	 	wy = db->textHeight;
	}
   
	pixelStepX = (double)maxX / (double)fd_bav_kurve->Grafik->w;
	pixelStepY = (double)maxY / (double)fd_bav_kurve->Grafik->h;

	if (bavhk_faltung) 
	{
		if (as_bmp==1)
			wb = maxX - wx - 2*db->textWidth;
		else
			wb -= wx + 2*db->textWidth;
	} 
	else
	{
		wb = trunc(pixelStepX*bavhk_fak_zeit*jdd);
	}
	wh = trunc(pixelStepY*bavhk_fak_hell*(bavhk_hmin-bavhk_hmax)*0.01);
	
	/* Helligkeitsachse und waagerechte Linien */
	draw_line(db,wx,wy,wx,wy+wh,FL_WHITE);
	draw_line(db,wx,wy+wh,wx+wb,wy+wh,FL_WHITE);
	draw_text(db,wx-2,wy-db->textHeight,FL_WHITE,"mag");
	hdif = (double)wh/(double)(bavhk_hmin-bavhk_hmax);
	for (i=bavhk_hmax/50; i<=bavhk_hmin/50; i++) 
	{
		y = trunc(hdif*(i*50-bavhk_hmax));
		if (y>=0) 
		{
			if (i%2==0) 
			{
				draw_line(db,wx-2,wy+y,wx+2,wy+y,FL_WHITE);
				sprintf(str,"%3i",i/2);
				draw_text(db,wx-3*db->textWidth-3,wy+y-db->textHeight/2,FL_WHITE,str);
			}
			xr = 0;
			while (trunc(xr)<wb) 
			{
				draw_line(db,wx+trunc(xr),wy+y,wx+trunc(xr+pixelStepX),wy+y,FL_WHITE);
				xr += 5*pixelStepX;
			}
		}
	}

	/* Anfangs- und Enddatum */
	if (bavhk_faltung) 
	{
		for (i=0; i<=10; i++) 
		{
			x = wx+(double)wb*i*0.1;
			draw_line(db, x,wy+wh-2, x,wy+wh+4, FL_WHITE);
			sprintf(str,"%i",i*10);
			draw_text(db, x-(1+strlen(str))*db->textWidth/2, wy+wh+4, FL_WHITE, str );
			
			/* gleich mit senkrechtem Grid */
			for (y=0; y<wh/5; y++)
				draw_line(db, x,wy+y*5+1, x,wy+y*5, FL_WHITE);
		}
		sprintf(str,"Faltung mit Periode= %g Tage", bavhk_periode );
		if (bavhk_spiegel)
			strcat(str,", Spiegelung an Phase 0.5");
		kommentarZeile++;
		draw_text(db, wx+3*db->textWidth, wy+wh+5+db->textHeight, FL_WHITE, str );
		kommentarZeile++;
		sprintf(str, "JD/ph0= %10.3f , JD/ph50= %10.3f", 
		             bavhk_faltung_jd-bavhk_faltungsphase*bavhk_periode,
		             bavhk_faltung_jd+(0.5-bavhk_faltungsphase)*bavhk_periode );
		draw_text(db, wx+3*db->textWidth, wy+wh+5+kommentarZeile*db->textHeight, FL_WHITE, str );
		kommentarZeile++;
	}
	else
	{
		draw_line(db,wx,wy+wh-2,wx,wy+wh+4,FL_WHITE);
		sprintf(str,"%10.3f",jd0);
		draw_text(db,wx-3*db->textWidth,wy+wh+db->textHeight/2,FL_WHITE,str);
		/*draw_line(db,wx+wb/2,wy+wh-2,wx+wb/2,wy+wh+4,FL_WHITE);*/
		draw_line(db,wx+wb,wy+wh-2,wx+wb,wy+wh+4,FL_WHITE);
		sprintf(str,"%10.3f",jd1);
		draw_text(db,wx+wb-8*db->textWidth,wy+wh+db->textHeight/2,FL_WHITE,str);
		kommentarZeile++;
	}
	
	/* Kommentar */
	kommentarN = 0;
	i = 0; 
	while (bavhk_kommentar[i]!='\0') 
	{
		int j=0;
		while (bavhk_kommentar[i+j]>31) 
		{	// extrahiere eine Zeile aus dem Kommentar
			str[j] = bavhk_kommentar[i+j];
			j++;
		}
		str[j] = '\0';
		if (j>0) 
		{
			// Kommentar analysieren fr das Berichts-File im BAV-Format, TL 20.04.05
			char* kExtrem = strstr(str,"Max");
			if (kExtrem==NULL)	kExtrem = strstr(str,"Min");
			if (kExtrem)
			{
				char* posJD = kExtrem+3;
				while (*posJD!='\0' && !isdigit(*posJD))
					posJD++;
				if (isdigit(*posJD))
				{
					kommentarMax[kommentarN] = (*(kExtrem+1)=='a');
					kommentarJD[kommentarN] = atof(posJD);
					while (isdigit(*posJD) || *posJD=='.')	posJD++;
					kommentarJDunsicher[kommentarN] = (*posJD==':');
					posJD = strstr(posJD,"mit");
					if (posJD)
					{
						posJD+=3;
						while (*posJD!='\0' && !isdigit(*posJD))	posJD++;
						strncpy(&kommentarHell[kommentarN][0],posJD,10);
						kommentarHell[kommentarN][9] = '\0';
						posJD = &kommentarHell[kommentarN][0];
						while (isdigit(*posJD) || *posJD=='.' || *posJD==':')	posJD++;
						if (posJD)	*posJD = '\0';
					}
					else
						kommentarHell[kommentarN][0] = '\0';
					kommentarBemerkung[kommentarN][0] = '\0';
					posJD = strstr(str,"mag");
					if (posJD)
					{
						posJD+=3;
						while (*posJD!='\0' && (*posJD==' ' || *posJD==',' || *posJD==':'))	posJD++;
						strncpy(&kommentarBemerkung[kommentarN][0],posJD,20);
						kommentarBemerkung[kommentarN][19] = '\0';
						posJD = strchr(&kommentarBemerkung[kommentarN][0],'\n');
						if (posJD)	*posJD = '\0';
					}
					kommentarN++;
				}
			}
			// Kommentarzeile im Bild darstellen
			i+=j;
			draw_text(db, wx,wy+wh+kommentarZeile*db->textHeight+db->textHeight/2, FL_WHITE, str);
			kommentarZeile++;
		} 
		else
			i++;
	}
	
	/* Datumslinien (senkrecht) */
	jdd = wb/jdd;
   /*
   xr = 0;
   do {
      yr = wh-pixelStepY;
      while (trunc(yr)>=0) 
      {
         draw_line(db,wx+trunc(xr),wy+trunc(yr),wx+trunc(xr),wy+trunc(yr+pixelStepY),FL_WHITE);
         yr -= 5*pixelStepY;
      }
      xr += 10*pixelStepX;
   } while (trunc(xr)<=wb);
   */

	if (!bavhk_faltung) 
	{
		jdGridStep = GetJDGridStep( jd1-jd0 );
		for (jGrid=jd0; jGrid<=jd1; jGrid+=jdGridStep) 
		{
			x = wx+jdd*(jGrid-jd0);
			draw_line(db, x, wy+wh-2, x, wy+wh+2, FL_WHITE);
			for (y=0; y<wh/5; y++)
				draw_line(db, x,wy+y*5+1, x,wy+y*5, FL_WHITE);
		}
		sprintf(str,"deltaT=%g",jdGridStep);
		draw_text(db,wx+wb/2-20,wy+wh,FL_WHITE,str);
	}

	/* Beobachtungen zeichnen */
	hzahl = 0;
	hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
		if ((jd0<=hellPtr->datum) && (hellPtr->datum<=jd1) && BitTest(besArt,hellPtr->art)) 
		{
			int x2 = -1;
			hzahl++;
			if (bavhk_faltung) 
         {
				double phase = fracp((hellPtr->datum-bavhk_faltung_jd)/bavhk_periode);
				phase -= bavhk_faltungsphase;
				if (phase<0)	phase += 1.0;
         	x = wx+trunc(phase*wb);
         	if (bavhk_spiegel)
         		x2 = wx+wb-1-trunc(phase*wb);
         } 
			else
	         x = wx+trunc(jdd*(hellPtr->datum-jd0));
			if (hellPtr->hell<10000) 
			{
				y = wy+trunc(hdif*(hellPtr->hell-bavhk_hmax));
				draw_circle_fill(db, x ,y, obsRadius, obsColor);
				if (x2>=0)
					draw_circle_fill(db, x2,y, obsRadius, obsColor);
         } 
         else 
         {
            if (mitNegativBeob) 
            {
					y = wy+trunc(hdif*(hellPtr->hell-10000-bavhk_hmax));
					SymbolKleiner(db, x,y, FL_RED );
					if (x2>=0)
						SymbolKleiner(db, x2,y, FL_RED );
            }
         }
      }
      hellPtr = hellPtr->next;
   }

	if (as_bmp>0) 
	{
		sprintf(str,"n= %i",hzahl);
		draw_text(db,textposNObsX,3*db->textHeight,FL_WHITE,str);

		CURSOR_WATCH(fd_bav_kurve->bav_kurve->window);
		if (as_bmp==1) 
		{	/* Print */
			FILE *fLst;
			fLst = OpenPrinter();
			wy = old_grafikDruckY+8;
			draw_rect(db,0,wy,maxX,2*maxY-1,FL_WHITE);
			wy += db->textHeight;
			wh = (2*maxY-4-wy)/db->textHeight;
			hellPtr = hellFirst;
			i = 0;
			while (hellPtr!=NULL) 
			{
				if ((jd0<=hellPtr->datum) && (hellPtr->datum<=jd1) && BitTest(besArt,hellPtr->art)) 
				{
					sprintf(str,"%10.3f: %c%5.2f\n",hellPtr->datum,
                                              (hellPtr->hell<10000) ? ' ' : '<',
                                              (hellPtr->hell%10000)*0.01);
					if (hzahl<wh) 
						draw_text(db,5*db->textWidth,wy+db->textHeight*i,FL_WHITE,str);
					else 
						draw_text(db,(5+40*(i/(hzahl/2)))*db->textWidth,wy+db->textHeight*(i%(hzahl/2)),FL_WHITE,str);
					i++;
				}
				hellPtr = hellPtr->next;
			}
			PrintScreen(fLst,db->pic->data,grafikDruckX,grafikDruckY,grafikDruckEinschubX,grafikDruckEinschubY);
			ClosePrinter(fLst);
			grafikDruckY = old_grafikDruckY;
		}	
		if (as_bmp==2) 
		{ 	/* Save */
			FILE *fLst;
			char fname[256];
			// File mit Einzelbeobachtungen in <stern>.lst
			sprintf(fname,"%s%s.lst",pfadAblage,diskLoad->name);
			fLst = fopen(fname,"w");
			hellPtr = hellFirst;
			while (hellPtr!=NULL) 
			{
				if ((jd0<=hellPtr->datum) && (hellPtr->datum<=jd1) && BitTest(besArt,hellPtr->art)) 
				{
					fprintf(fLst,"%10.3f: %c%5.2f\n",hellPtr->datum,
                                                (hellPtr->hell<10000) ? ' ' : '<',
                                                (hellPtr->hell%10000)*0.01);
				}
				hellPtr = hellPtr->next;
			}
			fclose(fLst);
			// File mit BAV Zusatzdaten in bav-<observer>.lst
			sprintf(fname,"%sbav-%s.lst",pfadAblage,(strlen(initialBAV)>0?initialBAV:"auswertung") );
			fLst = fopen(fname,"a");
			for (i=0; i<kommentarN; i++)
			{
				fprintf(fLst,"%-5.5s%-3.3s%c", diskLoad->index, diskLoad->stb, (kommentarMax[i]?'X':'N') );
				while (kommentarJD[i]>=100000)	kommentarJD[i] -= 100000;
				fprintf(fLst,"%07.1f",kommentarJD[i]);	// 24xxxxx.x
				fprintf(fLst,"%c",kommentarJDunsicher[i]?':':' ');
				fprintf(fLst,"%-6.6s",kommentarHell[i]);
				fprintf(fLst,"%03i%-3.3s            %-18.18s\n",hzahl,initialBAV,kommentarBemerkung[i]);
			}
			fclose(fLst);
			// Grafik selbst speichern
			draw_save(db);
		}
		CURSOR_NORM(fd_bav_kurve->bav_kurve->window);
	}
	draw_free(db);
	JulDatZone = oldZeitZone;
}

void BAV_HellKurve_Close ()
{
	fd_bav_kurve_wp.x = fd_bav_kurve->bav_kurve->x;
	fd_bav_kurve_wp.y = fd_bav_kurve->bav_kurve->y;
	fd_bav_kurve_wp.w = fd_bav_kurve->bav_kurve->w;
	fd_bav_kurve_wp.h = fd_bav_kurve->bav_kurve->h;
	fl_hide_form(fd_bav_kurve->bav_kurve);
}

void BAV_HellKurve_Reset ()
{
	struct RHell *hellPtr;
	char str[80];
	float jd0,jd1;

	if (stern.periode<0) 
	{	/* Nova */
		if (hellFirst!=NULL) 
		{
  	  		jd0 = hellFirst->datum;
			hellPtr = hellFirst;
  	  		while (hellPtr->next!=NULL) hellPtr = hellPtr->next;
  	  		jd1 = hellPtr->datum;
  	  		if (jd1-jd0>-stern.periode)
  	  			jd0 = jd1 + stern.periode;
		} 
		else 
		{
  			jd0 = stern.max_jd+stern.periode/10;
  			jd1 = Zeit2JulDat(GetDateStr(str));
  		}
	} 
	else 
	{ 	/* periodischer Stern */
		jd1 = Zeit2JulDat(GetDateStr(str));
		jd0 = jd1 - stern.periode;
	}	
	bavhk_dauer = jd1-jd0;
	bavhk_jd = jd0+bavhk_dauer/2;

	ZeitDatum( bavhk_jd, str );
	fl_set_input(fd_bav_kurve->Date,str);
	sprintf(str,"%6.2f",bavhk_dauer);
	fl_set_input( fd_bav_kurve->Days, str );

	bavhk_hmax = stern.hellMax;
	bavhk_hmin = stern.hellMin;
	hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
		if (hellPtr->hell<bavhk_hmax) bavhk_hmax = hellPtr->hell;
		if (hellPtr->hell%10000>bavhk_hmin) bavhk_hmin = hellPtr->hell%10000;
		hellPtr = hellPtr->next;
	}
	sprintf(str,"%4.2f",bavhk_hmax*0.01);
	fl_set_input(fd_bav_kurve->HellMax,str);
	sprintf(str,"%4.2f",bavhk_hmin*0.01);
	fl_set_input(fd_bav_kurve->HellMin,str);

	if (toupper(stern.typ[0])=='M') 
	{ 
  		bavhk_fak_zeit = 1; 
  		bavhk_fak_hell = 20; 
  	} 
	else 
	if ((stern.typ[0]=='R') && (stern.typ[1]=='R')) 
	{ 
  		bavhk_fak_zeit = 480; 
  		bavhk_fak_hell = 50; 
	} 
	else 
	{
		int y;
		float jdd = stern.periode;
		if (jdd>600) bavhk_fak_zeit = 0.1;
		else if (jdd>400) bavhk_fak_zeit = 0.2;
		else if (jdd>250) bavhk_fak_zeit = 0.3;
		else if (jdd>120) bavhk_fak_zeit = 0.5;
		else if (jdd>80) bavhk_fak_zeit = 1;
		else if (jdd>60) bavhk_fak_zeit = 1.5;
		else if (jdd>25) bavhk_fak_zeit = 2;
		else if (jdd>12) bavhk_fak_zeit = 5;
  		else if (jdd>7) bavhk_fak_zeit = 10;
  		else if (jdd>2.5) bavhk_fak_zeit = 20;
  		else if (jdd>1) bavhk_fak_zeit = 50;
  		else bavhk_fak_zeit = 100;
		y = (bavhk_hmin-bavhk_hmax)/100;
  		if (y>9) bavhk_fak_hell = 5;
  		else if (y>4) bavhk_fak_hell = 10;
  		else if (y>2) bavhk_fak_hell = 20;
  		else if (y==2) bavhk_fak_hell = 40;
  		else bavhk_fak_hell = 50;
	}
	bavhk_fak_hell *= 2;	// bessere Ausnutzung des Raums
	sprintf(str,"%4.1f",bavhk_fak_zeit);
	fl_set_input(fd_bav_kurve->Day_Pixel,str);
	sprintf(str,"%3.1f",bavhk_fak_hell);
	fl_set_input(fd_bav_kurve->mag_Pixel,str);

	bavhk_kommentar[0] = '\0';
	fl_set_input( fd_bav_kurve->Comment, "" );
	
//	fl_set_input( fd_bav_kurve->Faltung, 
	bavhk_periode = stern.periode;
	sprintf(str,"%f",bavhk_periode);
	fl_set_input( fd_bav_kurve->Period, str);
	bavhk_faltung_jd = stern.max_jd;
	ZeitDatum( bavhk_faltung_jd, str );
	fl_set_input(fd_bav_kurve->PhaseDate,str);
	sprintf(str,"%g",bavhk_faltungsphase);
	fl_set_input(fd_bav_kurve->PhaseOfDate,str);
}

void BAV_HellKurve_FormFaltung()
{
	fl_set_button( fd_bav_kurve->Faltung, bavhk_faltung );
	if (bavhk_faltung) 
	{
		fl_hide_object( fd_bav_kurve->Days );
		fl_show_object( fd_bav_kurve->DateFrom );
		fl_show_object( fd_bav_kurve->Period );
		fl_show_object( fd_bav_kurve->PhaseDate );
		fl_show_object( fd_bav_kurve->Mirror );
		fl_show_object( fd_bav_kurve->PhaseOfDate );
		fl_set_object_label( fd_bav_kurve->Date, LS(221) );
	}
	else 
	{
		fl_show_object( fd_bav_kurve->Days );
		fl_hide_object( fd_bav_kurve->DateFrom );
		fl_hide_object( fd_bav_kurve->Period );
		fl_hide_object( fd_bav_kurve->PhaseDate );
		fl_hide_object( fd_bav_kurve->Mirror );
		fl_hide_object( fd_bav_kurve->PhaseOfDate );
		fl_set_object_label( fd_bav_kurve->Date,LS_Date);
	}
}

void cb_BAV_HellKurve (FL_OBJECT *ob, long data)
{
	const XEvent *xe;
	
	switch (data) {
	case 1: /* Close */
		BAV_HellKurve_Close();
		return;
	case 2: /* Print */
		BAV_HellKurve_Paint(1);
		break;
	case 3: /* Save */
		BAV_HellKurve_Paint(2);
		break;
	case 4: /* Reset */
		BAV_HellKurve_Reset();
		break;
	case 9: /* switch One Phase */
		bavhk_faltung = fl_get_button( fd_bav_kurve->Faltung );
		BAV_HellKurve_FormFaltung();
		break;
	case 11: /* mm/Tag */
		bavhk_fak_zeit = atof( fl_get_input( fd_bav_kurve->Day_Pixel ) );
		break;
	case 12: /* mm/mag */
		bavhk_fak_hell = atof( fl_get_input(fd_bav_kurve->mag_Pixel) );
		break;
	case 13: /* Tage */
		bavhk_dauer = atof( fl_get_input(fd_bav_kurve->Days) );
		break;
	case 14: /* Datum */ 
		bavhk_jd = atof( fl_get_input(fd_bav_kurve->Date) );
		if (bavhk_jd<40)
			bavhk_jd = Zeit2JulDat( (char*)fl_get_input(fd_bav_kurve->Date) );
		break;
	case 15: /* HellMax */
		bavhk_hmax = (int)(100*atof( fl_get_input(fd_bav_kurve->HellMax) ) );
		break;
	case 16: /* HellMin */
		bavhk_hmin = (int)(100*atof( fl_get_input(fd_bav_kurve->HellMin) ) );
		break;
	case 17: /* Kommentar */
		strn0cpy( bavhk_kommentar, (char*)fl_get_input(fd_bav_kurve->Comment), sizeof(bavhk_kommentar)-1 );
		break;
	case 20: /* Period for One Phase */
		bavhk_periode = atof( fl_get_input(fd_bav_kurve->Period) );
		break;
	case 21: /* Date of center for One Phase */
		bavhk_faltung_jd = atof( fl_get_input(fd_bav_kurve->PhaseDate) );
		if (bavhk_faltung_jd<40)
			bavhk_faltung_jd = Zeit2JulDat( (char*)fl_get_input(fd_bav_kurve->PhaseDate) );
		break;
	case 22: /* Date from */
		bavhk_jd0 = atof( fl_get_input(fd_bav_kurve->DateFrom) );
		if (bavhk_jd0<40)
			bavhk_jd0 = Zeit2JulDat( (char*)fl_get_input(fd_bav_kurve->DateFrom) );
		break;
	case 23: /* Mirror Phase */
		bavhk_spiegel = fl_get_button( fd_bav_kurve->Mirror );
		break;
	case 24: /* Phase of date */
		bavhk_faltungsphase = atof( fl_get_input(fd_bav_kurve->PhaseOfDate) );
		break;
	case 99: /* Taste */
		xe = fl_last_event();
/*
		fprintf(stderr,"shortcut type=%i state=%i keycode=%i sum=%i\n",
              xe->type,xe->xkey.state,xe->xkey.keycode,xe->xkey.state*256+xe->xkey.keycode);
*/
		if (xe->type==3) 
		{
			switch (xe->xkey.state*256+xe->xkey.keycode) 
			{
			}
		}
	}
 	fl_redraw_form(fd_bav_kurve->bav_kurve);
	BAV_HellKurve_Paint(FALSE);
}

int BAV_HellKurve_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
{
	switch (event) 
	{
		case FL_DRAW:
			BAV_HellKurve_Paint(FALSE);
			break;
	}
	return 0;	
}

void BAV_HellKurve ()
{
	char str[80];

	if ((stern.hellZahl==0) && (stern.periode==0)) return;
	if ((fd_bav_kurve!=NULL) && (fd_bav_kurve->bav_kurve->visible))
		BAV_HellKurve_Close();
	else 
	{
		fd_bav_kurve = create_form_bav_kurve();	
		fl_set_form_minsize( fd_bav_kurve->bav_kurve, BAVHK_MINX, BAVHK_MINY );
	  	//fl_set_object_shortcut( fd_bav_kurve->shortcut, "", FALSE );
		fl_set_object_label(fd_bav_kurve->Reset, LS_Reset );
		fl_set_object_label(fd_bav_kurve->Close, LS_Close );
		fl_set_object_label(fd_bav_kurve->Save , LS_Save );
		fl_set_object_label(fd_bav_kurve->Print, LS_Print );
		fl_set_object_label(fd_bav_kurve->Comment, LS(212) );
		fl_set_object_label(fd_bav_kurve->Date , LS_Date );
		fl_set_object_label(fd_bav_kurve->Days , LS_Days );
		fl_set_object_label(fd_bav_kurve->Day_Pixel, LS(213) );
		fl_set_object_label(fd_bav_kurve->mag_Pixel , LS(215) );
		fl_set_object_label(fd_bav_kurve->HellMax, LS(216) );
		fl_set_object_label(fd_bav_kurve->HellMin, LS(217) );
		
		fl_set_object_label(fd_bav_kurve->Faltung, LS(218) );
		fl_set_object_label(fd_bav_kurve->Period , LS_Period );
		fl_set_object_label(fd_bav_kurve->PhaseDate, LS_Date );
		fl_set_object_label(fd_bav_kurve->PhaseOfDate, LS(222) );
		fl_set_object_label(fd_bav_kurve->DateFrom, LS(220) );
		fl_set_object_label(fd_bav_kurve->Mirror , LS(219) );

		fl_set_object_posthandler( fd_bav_kurve->Grafik, BAV_HellKurve_Posthandler );
		BAV_HellKurve_Reset();
	}  	                                           

	BAV_HellKurve_FormFaltung();
	sprintf(str,LS(210),stern.name);
	if (fd_bav_kurve_wp.x>0) 
	{
		fl_set_form_position(fd_bav_kurve->bav_kurve, fd_bav_kurve_wp.x,fd_bav_kurve_wp.y);
		fl_set_form_size    (fd_bav_kurve->bav_kurve, fd_bav_kurve_wp.w,fd_bav_kurve_wp.h);
	}
	fl_show_form( fd_bav_kurve->bav_kurve,
                 ((fd_bav_kurve_wp.x>0) ? FL_PLACE_POSITION : FL_PLACE_MOUSE | FL_FREE_SIZE),
                 FL_FULLBORDER,
                 str);
	fl_raise_form(fd_bav_kurve->bav_kurve);
	BAV_HellKurve_Paint(FALSE);
}

#endif // !NO_FORMS

